#ifndef ABSYN_HEADER
#define ABSYN_HEADER

#include<string>
#include<vector>
#include <memory>

//C++ Abstract Syntax Interface generated by the BNF Converter.

/********************   TypeDef Section    ********************/

typedef long long int Integer;
typedef char Char;
typedef double Double;
typedef std::string String;
typedef std::string Ident;




/********************   Forward Declarations    ********************/

#include "TypeDefs.h"


/********************   Visitor Interfaces    ********************/
class Visitor
{
public:
	virtual ~Visitor() {}
	virtual void visitProgram(Program *p) = 0;
	virtual void visitTopDef(TopDef *p) = 0;
	virtual void visitFunDef(FunDef *p) = 0;
	virtual void visitArg(Arg *p) = 0;
	virtual void visitClassDef(ClassDef *p) = 0;
	virtual void visitClassBlock(ClassBlock *p) = 0;
	virtual void visitClassBlockDef(ClassBlockDef *p) = 0;
	virtual void visitBlock(Block *p) = 0;
	virtual void visitStmt(Stmt *p) = 0;
	virtual void visitItem(Item *p) = 0;
	virtual void visitType(Type *p) = 0;
	virtual void visitExpr(Expr *p) = 0;
	virtual void visitAddOp(AddOp *p) = 0;
	virtual void visitMulOp(MulOp *p) = 0;
	virtual void visitRelOp(RelOp *p) = 0;
	virtual void visitProg(Prog *p) = 0;
	virtual void visitFnDef(FnDef *p) = 0;
	virtual void visitClDef(ClDef *p) = 0;
	virtual void visitFuncDef(FuncDef *p) = 0;
	virtual void visitAr(Ar *p) = 0;
	virtual void visitClassDefN(ClassDefN *p) = 0;
	virtual void visitClassDefE(ClassDefE *p) = 0;
	virtual void visitClassBl(ClassBl *p) = 0;
	virtual void visitClassMthd(ClassMthd *p) = 0;
	virtual void visitClassFld(ClassFld *p) = 0;
	virtual void visitBlk(Blk *p) = 0;
	virtual void visitEmpty(Empty *p) = 0;
	virtual void visitBStmt(BStmt *p) = 0;
	virtual void visitDecl(Decl *p) = 0;
	virtual void visitNoInit(NoInit *p) = 0;
	virtual void visitInit(Init *p) = 0;
	virtual void visitAss(Ass *p) = 0;
	virtual void visitIncr(Incr *p) = 0;
	virtual void visitDecr(Decr *p) = 0;
	virtual void visitRet(Ret *p) = 0;
	virtual void visitVRet(VRet *p) = 0;
	virtual void visitCond(Cond *p) = 0;
	virtual void visitCondElse(CondElse *p) = 0;
	virtual void visitWhile(While *p) = 0;
	virtual void visitSExp(SExp *p) = 0;
	virtual void visitForEach(ForEach *p) = 0;
	virtual void visitInt(Int *p) = 0;
	virtual void visitStr(Str *p) = 0;
	virtual void visitBool(Bool *p) = 0;
	virtual void visitVoid(Void *p) = 0;
	virtual void visitArray(Array *p) = 0;
	virtual void visitClassT(ClassT *p) = 0;
	virtual void visitFun(Fun *p) = 0;
	virtual void visitEVar(EVar *p) = 0;
	virtual void visitEIndexAcc(EIndexAcc *p) = 0;
	virtual void visitEClsMmbr(EClsMmbr *p) = 0;
	virtual void visitEApp(EApp *p) = 0;
	virtual void visitELitInt(ELitInt *p) = 0;
	virtual void visitELitTrue(ELitTrue *p) = 0;
	virtual void visitELitFalse(ELitFalse *p) = 0;
	virtual void visitEString(EString *p) = 0;
	virtual void visitENewArray(ENewArray *p) = 0;
	virtual void visitENewClass(ENewClass *p) = 0;
	virtual void visitNullCast(NullCast *p) = 0;
	virtual void visitNeg(Neg *p) = 0;
	virtual void visitNot(Not *p) = 0;
	virtual void visitEMul(EMul *p) = 0;
	virtual void visitEAdd(EAdd *p) = 0;
	virtual void visitERel(ERel *p) = 0;
	virtual void visitEAnd(EAnd *p) = 0;
	virtual void visitEOr(EOr *p) = 0;
	virtual void visitPlus(Plus *p) = 0;
	virtual void visitMinus(Minus *p) = 0;
	virtual void visitTimes(Times *p) = 0;
	virtual void visitDiv(Div *p) = 0;
	virtual void visitMod(Mod *p) = 0;
	virtual void visitLTH(LTH *p) = 0;
	virtual void visitLE(LE *p) = 0;
	virtual void visitGTH(GTH *p) = 0;
	virtual void visitGE(GE *p) = 0;
	virtual void visitEQU(EQU *p) = 0;
	virtual void visitNE(NE *p) = 0;
	virtual void visitListTopDef(ListTopDef *p) = 0;
	virtual void visitListArg(ListArg *p) = 0;
	virtual void visitListClassBlockDef(ListClassBlockDef *p) = 0;
	virtual void visitListStmt(ListStmt *p) = 0;
	virtual void visitListItem(ListItem *p) = 0;
	virtual void visitListType(ListType *p) = 0;
	virtual void visitListExpr(ListExpr *p) = 0;
	virtual void visitPIdent(PIdent *p) = 0;


	virtual void visitInteger(Integer x) = 0;
	virtual void visitChar(Char x) = 0;
	virtual void visitDouble(Double x) = 0;
	virtual void visitString(String x) = 0;
	virtual void visitIdent(Ident x) = 0;

};


class Visitable
{
public:
	int lineno;
	virtual ~Visitable() {}
	virtual void accept(Visitor *v) = 0;
};


/********************   Abstract Syntax Classes    ********************/

class Program : public Visitable
{
public:
	virtual Program *clone() const = 0;

};

class TopDef : public Visitable
{
public:
	virtual TopDef *clone() const = 0;

};

class FunDef : public Visitable
{
public:
	virtual FunDef *clone() const = 0;

};

class Arg : public Visitable
{
public:
	Type *type_;
	PIdent *pident_;
	virtual Arg *clone() const = 0;

};

class ClassDef : public Visitable
{
public:
	virtual PIdent *getName() const = 0;
	virtual PIdent *getParent() const = 0;
	virtual ClassBlock *getBlock() const = 0;
	virtual ClassDef *clone() const = 0;

};

class ClassBlock : public Visitable
{
public:
	virtual ClassBlock *clone() const = 0;

};

class ClassBlockDef : public Visitable
{
public:
	virtual ClassBlockDef *clone() const = 0;

};

class Block : public Visitable
{
public:
	std::weak_ptr<Binding> binding;
	std::weak_ptr<FunctionInfo> function;
	virtual Block *clone() const = 0;

};

class Stmt : public Visitable
{
public:
	virtual Stmt *clone() const = 0;

};

class Item : public Visitable
{
public:
	PIdent *pident_;
	Expr *expr_;

	Item(const Item &);
	Item &operator=(const Item &);
	Item(PIdent *p1, Expr *p2 = nullptr);
	~Item();
	virtual void accept(Visitor *v);
	virtual Item *clone() const;
	void swap(Item &);
};

class Type : public Visitable
{
public:
	Type() {};
	Type(const Type &other) : binding(other.binding) {};
	virtual void swap(Type &other) { std::swap(binding, other.binding); }
	std::weak_ptr<Binding> binding;
	virtual Type *clone() const = 0;
	virtual std::string printName() const = 0;
	virtual bool isEqual(Type const *f, bool sub = false) const = 0;
	template<typename T>
	bool isEqual(shared_ptr<T> f, bool sub = false) const { return isEqual((T*)&*f, sub); };
	bool operator==(Type const & f) const { return isEqual(&f); }
	bool operator!=(Type const & f) const { return !isEqual(&f); }
};

class Expr : public Visitable
{
public:
	virtual Expr *clone() const = 0;

};

class AddOp : public Visitable
{
public:
	virtual AddOp *clone() const = 0;

};

class MulOp : public Visitable
{
public:
	virtual MulOp *clone() const = 0;

};

class RelOp : public Visitable
{
public:
	virtual RelOp *clone() const = 0;

};

class VarInfo;
class PIdent : public Visitable
{
public:
	String string_;
	std::weak_ptr<VarInfo> var;

	PIdent(const PIdent &);
	PIdent &operator=(const PIdent &);
	PIdent(String p1, Integer p2 = 0);
	~PIdent();
	virtual void accept(Visitor *v);
	virtual PIdent *clone() const;
	void swap(PIdent &);

	operator std::string() const { return string_; }
};

class Prog : public Program
{
public:
	ListTopDef *listtopdef_;

	Prog(const Prog &);
	Prog &operator=(const Prog &);
	Prog(ListTopDef *p1);
	~Prog();
	virtual void accept(Visitor *v);
	virtual Prog *clone() const;
	void swap(Prog &);
};

class FnDef : public TopDef
{
public:
	FunDef *fundef_;

	FnDef(const FnDef &);
	FnDef &operator=(const FnDef &);
	FnDef(FunDef *p1);
	~FnDef();
	virtual void accept(Visitor *v);
	virtual FnDef *clone() const;
	void swap(FnDef &);
};

class ClDef : public TopDef
{
public:
	ClassDef *classdef_;

	ClDef(const ClDef &);
	ClDef &operator=(const ClDef &);
	ClDef(ClassDef *p1);
	~ClDef();
	virtual void accept(Visitor *v);
	virtual ClDef *clone() const;
	void swap(ClDef &);
};

class FuncDef : public FunDef
{
public:
	Type *type_;
	PIdent *pident_;
	ListArg *listarg_;
	Block *block_;

	FuncDef(const FuncDef &);
	FuncDef &operator=(const FuncDef &);
	FuncDef(Type *p1, PIdent *p2, ListArg *p3, Block *p4);
	~FuncDef();
	virtual void accept(Visitor *v);
	virtual FuncDef *clone() const;
	void swap(FuncDef &);
};

class Ar : public Arg
{
public:

	Ar(const Ar &);
	Ar &operator=(const Ar &);
	Ar(Type *p1, PIdent *p2);
	~Ar();
	virtual void accept(Visitor *v);
	virtual Ar *clone() const;
	void swap(Ar &);
};

class ClassDefN : public ClassDef
{
public:
	PIdent *pident_;
	ClassBlock *classblock_;

	ClassDefN(const ClassDefN &);
	ClassDefN &operator=(const ClassDefN &);
	ClassDefN(PIdent *p1, ClassBlock *p2);
	~ClassDefN();
	virtual void accept(Visitor *v);
	virtual ClassDefN *clone() const;
	void swap(ClassDefN &);

	virtual PIdent *getName() const { return pident_; };
	virtual PIdent *getParent() const { return NULL; };
	virtual ClassBlock *getBlock() const { return classblock_; };
};

class ClassDefE : public ClassDef
{
public:
	PIdent *pident_1;
	PIdent *pident_2;
	ClassBlock *classblock_;

	ClassDefE(const ClassDefE &);
	ClassDefE &operator=(const ClassDefE &);
	ClassDefE(PIdent *p1, PIdent *p2, ClassBlock *p3);
	~ClassDefE();
	virtual void accept(Visitor *v);
	virtual ClassDefE *clone() const;
	void swap(ClassDefE &);

	virtual PIdent *getName() const { return pident_1; };
	virtual PIdent *getParent() const { return pident_2; };
	virtual ClassBlock *getBlock() const { return classblock_; };
};

class ClassBl : public ClassBlock
{
public:
	ListClassBlockDef *listclassblockdef_;

	ClassBl(const ClassBl &);
	ClassBl &operator=(const ClassBl &);
	ClassBl(ListClassBlockDef *p1);
	~ClassBl();
	virtual void accept(Visitor *v);
	virtual ClassBl *clone() const;
	void swap(ClassBl &);
};

class ClassMthd : public ClassBlockDef
{
public:
	FunDef *fundef_;

	ClassMthd(const ClassMthd &);
	ClassMthd &operator=(const ClassMthd &);
	ClassMthd(FunDef *p1);
	~ClassMthd();
	virtual void accept(Visitor *v);
	virtual ClassMthd *clone() const;
	void swap(ClassMthd &);
};

class ClassFld : public ClassBlockDef
{
public:
	Type *type_;
	ListItem *listitem_;

	ClassFld(const ClassFld &);
	ClassFld &operator=(const ClassFld &);
	ClassFld(Type *p1, ListItem *p2);
	~ClassFld();
	virtual void accept(Visitor *v);
	virtual ClassFld *clone() const;
	void swap(ClassFld &);
};

class Blk : public Block
{
public:
	ListStmt *liststmt_;

	Blk(const Blk &);
	Blk &operator=(const Blk &);
	Blk(ListStmt *p1);
	~Blk();
	virtual void accept(Visitor *v);
	virtual Blk *clone() const;
	void swap(Blk &);
};

class Empty : public Stmt
{
public:

	Empty(const Empty &);
	Empty &operator=(const Empty &);
	Empty();
	~Empty();
	virtual void accept(Visitor *v);
	virtual Empty *clone() const;
	void swap(Empty &);
};

class BStmt : public Stmt
{
public:
	Block *block_;

	BStmt(const BStmt &);
	BStmt &operator=(const BStmt &);
	BStmt(Block *p1);
	~BStmt();
	virtual void accept(Visitor *v);
	virtual BStmt *clone() const;
	void swap(BStmt &);
};

class Decl : public Stmt
{
public:
	Type *type_;
	ListItem *listitem_;

	Decl(const Decl &);
	Decl &operator=(const Decl &);
	Decl(Type *p1, ListItem *p2);
	~Decl();
	virtual void accept(Visitor *v);
	virtual Decl *clone() const;
	void swap(Decl &);
};

class Ass : public Stmt
{
public:
	Expr *expr_1;
	Expr *expr_2;

	Ass(const Ass &);
	Ass &operator=(const Ass &);
	Ass(Expr *p1, Expr *p2);
	~Ass();
	virtual void accept(Visitor *v);
	virtual Ass *clone() const;
	void swap(Ass &);
};

class Incr : public Stmt
{
public:
	Expr *expr_;

	Incr(const Incr &);
	Incr &operator=(const Incr &);
	Incr(Expr *p1);
	~Incr();
	virtual void accept(Visitor *v);
	virtual Incr *clone() const;
	void swap(Incr &);
};

class Decr : public Stmt
{
public:
	Expr *expr_;

	Decr(const Decr &);
	Decr &operator=(const Decr &);
	Decr(Expr *p1);
	~Decr();
	virtual void accept(Visitor *v);
	virtual Decr *clone() const;
	void swap(Decr &);
};

class Ret : public Stmt
{
public:
	Expr *expr_;

	Ret(const Ret &);
	Ret &operator=(const Ret &);
	Ret(Expr *p1);
	~Ret();
	virtual void accept(Visitor *v);
	virtual Ret *clone() const;
	void swap(Ret &);
};

class VRet : public Stmt
{
public:

	VRet(const VRet &);
	VRet &operator=(const VRet &);
	VRet();
	~VRet();
	virtual void accept(Visitor *v);
	virtual VRet *clone() const;
	void swap(VRet &);
};

class Cond : public Stmt
{
public:
	Expr *expr_;
	Stmt *stmt_;

	Cond(const Cond &);
	Cond &operator=(const Cond &);
	Cond(Expr *p1, Stmt *p2);
	~Cond();
	virtual void accept(Visitor *v);
	virtual Cond *clone() const;
	void swap(Cond &);
};

class CondElse : public Stmt
{
public:
	Expr *expr_;
	Stmt *stmt_1;
	Stmt *stmt_2;

	CondElse(const CondElse &);
	CondElse &operator=(const CondElse &);
	CondElse(Expr *p1, Stmt *p2, Stmt *p3);
	~CondElse();
	virtual void accept(Visitor *v);
	virtual CondElse *clone() const;
	void swap(CondElse &);
};

class While : public Stmt
{
public:
	Expr *expr_;
	Stmt *stmt_;

	While(const While &);
	While &operator=(const While &);
	While(Expr *p1, Stmt *p2);
	~While();
	virtual void accept(Visitor *v);
	virtual While *clone() const;
	void swap(While &);
};

class SExp : public Stmt
{
public:
	Expr *expr_;

	SExp(const SExp &);
	SExp &operator=(const SExp &);
	SExp(Expr *p1);
	~SExp();
	virtual void accept(Visitor *v);
	virtual SExp *clone() const;
	void swap(SExp &);
};

class ForEach : public Stmt
{
public:
	Type *type_;
	PIdent *pident_;
	Expr *expr_;
	Stmt *stmt_;

	ForEach(const ForEach &);
	ForEach &operator=(const ForEach &);
	ForEach(Type *p1, PIdent *p2, Expr *p3, Stmt *p4);
	~ForEach();
	virtual void accept(Visitor *v);
	virtual ForEach *clone() const;
	void swap(ForEach &);
};


class Int : public Type
{
public:
	Int(const Int &);
	Int &operator=(const Int &);
	Int();
	~Int();
	virtual void accept(Visitor *v);
	virtual Int *clone() const;
	void swap(Int &);
	std::string printName() const { return "int"; }
	bool isEqual(Type const *other, bool sub = false) const {
		if (dynamic_cast<const Int*>(other))
			return true;
		return false;
	}
};

class Str : public Type
{
public:
	Str(const Str &);
	Str &operator=(const Str &);
	Str();
	~Str();
	virtual void accept(Visitor *v);
	virtual Str *clone() const;
	void swap(Str &);
	std::string printName() const { return "string"; }
	bool isEqual(Type const *other, bool sub = false) const {
		if (dynamic_cast<const Str*>(other))
			return true;
		return false;
	}
};

class Bool : public Type
{
public:

	Bool(const Bool &);
	Bool &operator=(const Bool &);
	Bool();
	~Bool();
	virtual void accept(Visitor *v);
	virtual Bool *clone() const;
	void swap(Bool &);
	std::string printName() const { return "bool"; }
	bool isEqual(Type const *other, bool sub = false) const {
		if (dynamic_cast<const Bool*>(other))
			return true;
		return false;
	}
};

class Void : public Type
{
public:

	Void(const Void &);
	Void &operator=(const Void &);
	Void();
	~Void();
	virtual void accept(Visitor *v);
	virtual Void *clone() const;
	void swap(Void &);
	std::string printName() const { return "void"; }
	bool isEqual(Type const *other, bool sub = false) const {
		if (dynamic_cast<const Void*>(other))
			return true;
		return false;
	}
};

class Array : public Type
{
public:
	Type *type_;

	Array(const Array &);
	Array &operator=(const Array &);
	Array(Type *p1);
	Array() : type_(nullptr) {};
	~Array();
	virtual void accept(Visitor *v);
	virtual Array *clone() const;
	void swap(Array &);
	std::string printName() const { return type_ ? type_->printName() + "[]" : "array"; }
	bool isEqual(Type const *other, bool sub = false) const {
		if (const Array* casted = dynamic_cast<const Array*>(other))
			return type_->isEqual(casted->type_, sub);
		return false;
	}
};

class ClassT : public Type
{
public:
	PIdent *pident_;

	ClassT(const ClassT &);
	ClassT &operator=(const ClassT &);
	ClassT(PIdent *p1);
	ClassT() : pident_(nullptr) {};
	~ClassT();
	virtual void accept(Visitor *v);
	virtual ClassT *clone() const;
	void swap(ClassT &);
	std::string printName() const { return pident_ ? ("class " + pident_->string_) : "class"; }
	bool isEqual(Type const *other, bool sub = false) const;
};

class Fun : public Type
{
public:
	Type *type_;
	ListType *listtype_;

	Fun(const Fun &);
	Fun() : type_(nullptr), listtype_(nullptr) {};
	Fun &operator=(const Fun &);
	Fun(Type *p1, ListType *p2);
	~Fun();
	virtual void accept(Visitor *v);
	virtual Fun *clone() const;
	void swap(Fun &);
	std::string printName() const;
	bool isEqual(Type const *other, bool sub = false) const;
};

class EVar : public Expr
{
public:
	PIdent *pident_;

	EVar(const EVar &);
	EVar &operator=(const EVar &);
	EVar(PIdent *p1);
	~EVar();
	virtual void accept(Visitor *v);
	virtual EVar *clone() const;
	void swap(EVar &);
};

class EIndexAcc : public Expr
{
public:
	Expr *expr_1;
	Expr *expr_2;

	EIndexAcc(const EIndexAcc &);
	EIndexAcc &operator=(const EIndexAcc &);
	EIndexAcc(Expr *p1, Expr *p2);
	~EIndexAcc();
	virtual void accept(Visitor *v);
	virtual EIndexAcc *clone() const;
	void swap(EIndexAcc &);
};

class EClsMmbr : public Expr
{
public:
	Expr *expr_;
	PIdent *pident_;

	EClsMmbr(const EClsMmbr &);
	EClsMmbr &operator=(const EClsMmbr &);
	EClsMmbr(Expr *p1, PIdent *p2);
	~EClsMmbr();
	virtual void accept(Visitor *v);
	virtual EClsMmbr *clone() const;
	void swap(EClsMmbr &);
};

class EApp : public Expr
{
public:
	Expr *expr_;
	ListExpr *listexpr_;

	EApp(const EApp &);
	EApp &operator=(const EApp &);
	EApp(Expr *p1, ListExpr *p2);
	~EApp();
	virtual void accept(Visitor *v);
	virtual EApp *clone() const;
	void swap(EApp &);
};


class ELitInt : public Expr
{
public:
	Integer integer_;

	ELitInt(const ELitInt &);
	ELitInt &operator=(const ELitInt &);
	ELitInt(Integer p1);
	~ELitInt();
	virtual void accept(Visitor *v);
	virtual ELitInt *clone() const;
	void swap(ELitInt &);
};

class ELitTrue : public Expr
{
public:

	ELitTrue(const ELitTrue &);
	ELitTrue &operator=(const ELitTrue &);
	ELitTrue();
	~ELitTrue();
	virtual void accept(Visitor *v);
	virtual ELitTrue *clone() const;
	void swap(ELitTrue &);
};

class ELitFalse : public Expr
{
public:

	ELitFalse(const ELitFalse &);
	ELitFalse &operator=(const ELitFalse &);
	ELitFalse();
	~ELitFalse();
	virtual void accept(Visitor *v);
	virtual ELitFalse *clone() const;
	void swap(ELitFalse &);
};

class EString : public Expr
{
public:
	String string_;

	EString(const EString &);
	EString &operator=(const EString &);
	EString(String p1);
	~EString();
	virtual void accept(Visitor *v);
	virtual EString *clone() const;
	void swap(EString &);
};

class ENewArray : public Expr
{
public:
	Type *type_;
	Expr *expr_;

	ENewArray(const ENewArray &);
	ENewArray &operator=(const ENewArray &);
	ENewArray(Type *p1, Expr *p2);
	~ENewArray();
	virtual void accept(Visitor *v);
	virtual ENewArray *clone() const;
	void swap(ENewArray &);
};

class ENewClass : public Expr
{
public:
	PIdent *pident_;

	ENewClass(const ENewClass &);
	ENewClass &operator=(const ENewClass &);
	ENewClass(PIdent *p1);
	~ENewClass();
	virtual void accept(Visitor *v);
	virtual ENewClass *clone() const;
	void swap(ENewClass &);
};

class NullCast : public Expr
{
public:
	Expr *expr_;

	NullCast(const NullCast &);
	NullCast &operator=(const NullCast &);
	NullCast(Expr *p1);
	~NullCast();
	virtual void accept(Visitor *v);
	virtual NullCast *clone() const;
	void swap(NullCast &);
};

class Neg : public Expr
{
public:
	Expr *expr_;

	Neg(const Neg &);
	Neg &operator=(const Neg &);
	Neg(Expr *p1);
	~Neg();
	virtual void accept(Visitor *v);
	virtual Neg *clone() const;
	void swap(Neg &);
};

class Not : public Expr
{
public:
	Expr *expr_;

	Not(const Not &);
	Not &operator=(const Not &);
	Not(Expr *p1);
	~Not();
	virtual void accept(Visitor *v);
	virtual Not *clone() const;
	void swap(Not &);
};

class EMul : public Expr
{
public:
	Expr *expr_1;
	MulOp *mulop_;
	Expr *expr_2;

	EMul(const EMul &);
	EMul &operator=(const EMul &);
	EMul(Expr *p1, MulOp *p2, Expr *p3);
	~EMul();
	virtual void accept(Visitor *v);
	virtual EMul *clone() const;
	void swap(EMul &);
};

class EAdd : public Expr
{
public:
	Expr *expr_1;
	AddOp *addop_;
	Expr *expr_2;

	EAdd(const EAdd &);
	EAdd &operator=(const EAdd &);
	EAdd(Expr *p1, AddOp *p2, Expr *p3);
	~EAdd();
	virtual void accept(Visitor *v);
	virtual EAdd *clone() const;
	void swap(EAdd &);
};

class ERel : public Expr
{
public:
	Expr *expr_1;
	RelOp *relop_;
	Expr *expr_2;

	ERel(const ERel &);
	ERel &operator=(const ERel &);
	ERel(Expr *p1, RelOp *p2, Expr *p3);
	~ERel();
	virtual void accept(Visitor *v);
	virtual ERel *clone() const;
	void swap(ERel &);
};

class EAnd : public Expr
{
public:
	Expr *expr_1;
	Expr *expr_2;

	EAnd(const EAnd &);
	EAnd &operator=(const EAnd &);
	EAnd(Expr *p1, Expr *p2);
	~EAnd();
	virtual void accept(Visitor *v);
	virtual EAnd *clone() const;
	void swap(EAnd &);
};

class EOr : public Expr
{
public:
	Expr *expr_1;
	Expr *expr_2;

	EOr(const EOr &);
	EOr &operator=(const EOr &);
	EOr(Expr *p1, Expr *p2);
	~EOr();
	virtual void accept(Visitor *v);
	virtual EOr *clone() const;
	void swap(EOr &);
};

class Plus : public AddOp
{
public:

	Plus(const Plus &);
	Plus &operator=(const Plus &);
	Plus();
	~Plus();
	virtual void accept(Visitor *v);
	virtual Plus *clone() const;
	void swap(Plus &);
};

class Minus : public AddOp
{
public:

	Minus(const Minus &);
	Minus &operator=(const Minus &);
	Minus();
	~Minus();
	virtual void accept(Visitor *v);
	virtual Minus *clone() const;
	void swap(Minus &);
};

class Times : public MulOp
{
public:

	Times(const Times &);
	Times &operator=(const Times &);
	Times();
	~Times();
	virtual void accept(Visitor *v);
	virtual Times *clone() const;
	void swap(Times &);
};

class Div : public MulOp
{
public:

	Div(const Div &);
	Div &operator=(const Div &);
	Div();
	~Div();
	virtual void accept(Visitor *v);
	virtual Div *clone() const;
	void swap(Div &);
};

class Mod : public MulOp
{
public:

	Mod(const Mod &);
	Mod &operator=(const Mod &);
	Mod();
	~Mod();
	virtual void accept(Visitor *v);
	virtual Mod *clone() const;
	void swap(Mod &);
};

class LTH : public RelOp
{
public:

	LTH(const LTH &);
	LTH &operator=(const LTH &);
	LTH();
	~LTH();
	virtual void accept(Visitor *v);
	virtual LTH *clone() const;
	void swap(LTH &);
};

class LE : public RelOp
{
public:

	LE(const LE &);
	LE &operator=(const LE &);
	LE();
	~LE();
	virtual void accept(Visitor *v);
	virtual LE *clone() const;
	void swap(LE &);
};

class GTH : public RelOp
{
public:

	GTH(const GTH &);
	GTH &operator=(const GTH &);
	GTH();
	~GTH();
	virtual void accept(Visitor *v);
	virtual GTH *clone() const;
	void swap(GTH &);
};

class GE : public RelOp
{
public:

	GE(const GE &);
	GE &operator=(const GE &);
	GE();
	~GE();
	virtual void accept(Visitor *v);
	virtual GE *clone() const;
	void swap(GE &);
};

class EQU : public RelOp
{
public:

	EQU(const EQU &);
	EQU &operator=(const EQU &);
	EQU();
	~EQU();
	virtual void accept(Visitor *v);
	virtual EQU *clone() const;
	void swap(EQU &);
};

class NE : public RelOp
{
public:

	NE(const NE &);
	NE &operator=(const NE &);
	NE();
	~NE();
	virtual void accept(Visitor *v);
	virtual NE *clone() const;
	void swap(NE &);
};



class ListTopDef : public Visitable, public std::vector<TopDef*>
{
public:
	virtual void accept(Visitor *v);
	virtual ListTopDef *clone() const;
};

class ListArg : public Visitable, public std::vector<Arg*>
{
public:
	virtual void accept(Visitor *v);
	virtual ListArg *clone() const;
};

class ListClassBlockDef : public Visitable, public std::vector<ClassBlockDef*>
{
public:
	virtual void accept(Visitor *v);
	virtual ListClassBlockDef *clone() const;
};

class ListStmt : public Visitable, public std::vector<Stmt*>
{
public:
	virtual void accept(Visitor *v);
	virtual ListStmt *clone() const;
};

class ListItem : public Visitable, public std::vector<Item*>
{
public:
	virtual void accept(Visitor *v);
	virtual ListItem *clone() const;
};

class ListType : public Visitable, public std::vector<Type*>
{
public:
	virtual void accept(Visitor *v);
	virtual ListType *clone() const;
};

class ListExpr : public Visitable, public std::vector<Expr*>
{
public:
	virtual void accept(Visitor *v);
	virtual ListExpr *clone() const;
};



#endif
