#include <stdio.h>
#include <string.h>
#include "Parser.h"
#include "Printer.h"
#include "Absyn.h"
// #include "Compiler.h"
#include "ParseError.h"
#include "TypeCheck.h"
#include <fstream>
#include <iostream>
#include <filesystem>

std::filesystem::path binaryPath;

void usage() {
	printf("usage: Call with one of the following argument combinations:\n");
	printf("\t--help\t\tDisplay this help message.\n");
	printf("\t(no arguments)	Parse stdin silently.\n");
	printf("\t(files)\t\tParse content of files silently.\n");
	printf("\t-v (files)\tVerbose mode. Parse content of files verbosely.\n");
}

const int CONTEXT_LINES = 3;

int main(int argc, char ** argv)
{
	FILE *input;
	int quiet = 0;
	char *filename = NULL;

	if (argc > 1) {
		if (strcmp(argv[1], "-v") == 0) {
			if (argc > 2) {
				filename = argv[2];
			} else {
				input = stdin;
			}
		} else {
			quiet = 1;
			filename = argv[1];
		}
	}

	if (filename) {
		input = fopen(filename, "r");
		if (!input) {
			usage();
			exit(1);
		}
	} else input = stdin;

	string source;
	while (int c = fgetc(input)) {
		if (c == EOF) break;
		source += c;
	}

	/* The default entry point is used. For other options see Parser.H */
	Program *parse_tree = pProgram(source.data());
	if (!parse_tree) {
		fprintf(stderr, "Parser error, invalid syntax!\n");
		return 1;
	}
	fclose(input);

	if (!quiet) {
		fprintf(stderr, "\n[Abstract Syntax]\n");
		ShowAbsyn s;
		fprintf(stderr, "%s\n\n", s.show(parse_tree));
		fprintf(stderr, "[Linearized Tree]\n");
		PrintAbsyn p;
		fprintf(stderr, "%s\n\n", p.print(parse_tree));
	}

	binaryPath = argv[0];
	std::filesystem::path file(filename ? filename : "Latte");
	try {
		TypeCheck checker;
		checker.check(parse_tree);
	} catch (ParseError const &e) {
		std::cerr << e.what() << std::endl;
		if (e.line != -1) {
			auto ss = std::stringstream{source};
			int l = 1;
			int ll = max(max(to_string(e.line - CONTEXT_LINES).length(), to_string(e.line + CONTEXT_LINES).length()), 3lu);
		    for (std::string line; std::getline(ss, line, '\n');l++) {
		    	auto diff = abs(e.line - l);
		    	if (diff <= CONTEXT_LINES) {
		    		if (!diff) {
		    			cout << "--> ";
		    			for (int i = 2; i < ll; i++) cout << ' ';
	    				cout << line << endl;
		    		} else {
		    			cout << to_string(l) << ": ";
		    			for (int i = to_string(l).length(); i < ll; i++) cout << ' ';
			    		cout << line << endl;
		    		}
		    	}
		    }
		}
		return 1;
	}
	std::cout << "OK!" << endl;
	/*Compiler c(file);
	std::string out = c.compile(parse_tree);

	if (!filename) {
		printf("%s\n", out.data());
		return 0;
	}

	file.replace_extension(Compiler::extension);

	std::ofstream ll(file);
	if (!ll) {
		fprintf(stderr, "Unable to open %s\n", file.c_str());
		return 2;
	}
	ll << out;
	ll.close();

	Compiler::externalCommand(file);*/
}

