#include "ParseError.h"
#include "Info.h"
#include "Printer.h"
#include <sstream>
using namespace std;

ParseError::ParseError(string reason, int line) : runtime_error("ParseError"), line(line)
{
	msg = reason;
	if (line != -1)
		msg += "\nAt line " + to_string(line);
}

ParseError::ParseError(string reason, Visitable *expr) : runtime_error("ParseError")
{
	line = expr->lineno;
	msg = reason + "\nIn expression at line " + to_string(line) + ": " + PrintAbsyn().print(expr);
}

ParseError::ParseError(string reason, VarInfo &var) : runtime_error("ParseError") {
	msg = reason + "\nIn " + var.describe();
	line = var.lineLocation;
}

RedefinedError::RedefinedError(PIdent *ident, VarInfoPtr orig) {
	stringstream ss;
	ss << "Variable \"" << ident->string_ << "\" at line " << ident->lineno << " redeclared! First declaration at line " << orig->lineLocation;
	msg = ss.str();
	line = ident->lineno;
}

UndefinedError::UndefinedError(PIdent *ident) {
	stringstream ss;
	ss << "Undefined reference to \"" << ident->string_ << "\" at line " << ident->lineno;
	msg = ss.str();
	line = ident->lineno;
}

InvalidTypeError::InvalidTypeError(Type &expected, vector<shared_ptr<Type>> received, Visitable *expr) {
	stringstream ss;
	ss << "Invalid expression type: Expected " << expected.printName() << ", instead received ";
	bool fst = true;
	for (auto i : received) {
		if (fst) fst = false;
		else ss << ", ";
		ss << i->printName();
	}
	ss << ".";
	if (expr) {
		line = expr->lineno;
		ss << "\nIn expression at line " << line << ": " << PrintAbsyn().print(expr);
	}
	msg = ss.str();
}

InvalidTypeError::InvalidTypeError(Type &expected, vector<Type*> received, Visitable *expr) {
	stringstream ss;
	ss << "Invalid expression type: Expected " << expected.printName() << ", instead received ";
	bool fst = true;
	for (auto i : received) {
		if (fst) fst = false;
		else ss << ", ";
		ss << i->printName();
	}
	ss << ".";
	if (expr) {
		line = expr->lineno;
		ss << "\nIn expression at line " << line << ": " << PrintAbsyn().print(expr);
	}
	msg = ss.str();
	line = expr->lineno;
}
